/**************************************************************************
filename:   EPaper.js
author:     wangzhiyong
created:    2015/10/22
purpose:    基于htm5的手写签名组件
**************************************************************************/

/**
* 调用时最好设定好canvas的width和height属性，注意不是style里的width和height。
* 如果用style设置canvas的大小，EPaper也会重新设置好canvas的width和height,不过在
* 个别浏览器上会出现渲染问题
* @constructor
* @export
*/
function HandWrite() {
    function bytes2LongLong(array){
			if(!array) return -1;
			var high = array.length > 4 ? array.slice(0, array.length - 4) : [];
			var low = array.length > 4 ? array.slice(array.length - 4, array.length) : array;
			var highValue = 0;
			for(var i = high.length - 1; i >= 0; i--){
				highValue = highValue | ((high[i] & 0xFF) << ((high.length - 1 - i) * 8));
			}

			var lowValue = 0;
			for(var i = low.length - 1; i >= 0; i--){
				lowValue = lowValue | ((low[i] & 0xFF) << ((low.length - 1 - i) * 8));
			}

	    return highValue * 4294967295 + lowValue;
		}

    function getSubArray(array, start, end){
				if(!array || end >  array.lenght || start < 0)return -1;
				return array.slice(start,end);
		}

	function bytes2String(array){
    			if(!array) return -1;
    			return String.fromCharCode.apply(String, array);
    		}
    function stringToByte(str) {
        var bytes = new Array();
        var len, c;
        len = str.length;
        for(var i = 0; i < len; i++) {
            c = str.charCodeAt(i);
            if(c >= 0x010000 && c <= 0x10FFFF) {
                bytes.push(((c >> 18) & 0x07) | 0xF0);
                bytes.push(((c >> 12) & 0x3F) | 0x80);
                bytes.push(((c >> 6) & 0x3F) | 0x80);
                bytes.push((c & 0x3F) | 0x80);
            } else if(c >= 0x000800 && c <= 0x00FFFF) {
                bytes.push(((c >> 12) & 0x0F) | 0xE0);
                bytes.push(((c >> 6) & 0x3F) | 0x80);
                bytes.push((c & 0x3F) | 0x80);
            } else if(c >= 0x000080 && c <= 0x0007FF) {
                bytes.push(((c >> 6) & 0x1F) | 0xC0);
                bytes.push((c & 0x3F) | 0x80);
            } else {
                bytes.push(c & 0xFF);
            }
        }
        return bytes;


    }

    // public method
    this.paint =function(data){
			if(!data) return -1;
			var allPoints = [] ;
			var version = bytes2LongLong(getSubArray(data, 0, 2));
			var explorerInfoLength = bytes2LongLong(getSubArray(data, 2, 4));
			var explorerInfo = bytes2String(getSubArray(data, 4, 4 + explorerInfoLength));
			var platformInfoLength = bytes2LongLong(getSubArray(data, 4 + explorerInfoLength, 6 + explorerInfoLength));
			var platformInfo = bytes2String(getSubArray(data, 6 + explorerInfoLength, 6 + explorerInfoLength + platformInfoLength));
			var strokeCount = bytes2LongLong(getSubArray(data,6 + explorerInfoLength + platformInfoLength, 8 + explorerInfoLength + platformInfoLength));
			var startTime = bytes2LongLong(getSubArray(data, 8 + explorerInfoLength + platformInfoLength, 14 + explorerInfoLength + platformInfoLength));
			if(version == -1 || explorerInfoLength == -1 || explorerInfo == -1 || platformInfoLength == -1 || platformInfo == -1 || strokeCount == -1 || startTime == -1){
				return -1;
			}
			if(strokeCount == 0) return 0;
			var currentIndex = 14 + explorerInfoLength + platformInfoLength;
			for (var i = 0; i < strokeCount; i++) {
                var strokeTime = bytes2LongLong(getSubArray(data, currentIndex, currentIndex + 2));
                var red = bytes2LongLong(getSubArray(data, currentIndex + 2, currentIndex + 3));
                var green = bytes2LongLong(getSubArray(data, currentIndex + 3, currentIndex + 4));
                var blue = bytes2LongLong(getSubArray(data, currentIndex + 4, currentIndex + 5));
                var strokePenSize = bytes2LongLong(getSubArray(data, currentIndex + 5, currentIndex + 6));
                var strokePointsCount = bytes2LongLong(getSubArray(data, currentIndex + 6, currentIndex + 8));
                if (strokeTime == -1 || red == -1 || green == -1 || blue == -1 || strokePointsCount == -1) {
                    return -1;
                }
                currentIndex = currentIndex + 8;
                var points = [];
                var xArray = [];
                var yArray = [];
                var tArray = [];
                var j = 0;
                while (j < strokePointsCount) {
                    var xBytes = getSubArray(data, currentIndex, currentIndex + 2);
                    if (xBytes[0] == -1) {
                        for (var k = 0; k < xBytes[1] - 1; k++) {
                            xArray.push(xArray[xArray.length - 1]);
                        }
                        j += k;
                    } else {
                        xArray.push(bytes2LongLong(xBytes));
                        j++;
                    }
                    currentIndex = currentIndex + 2;
                }

                j = 0;
                while (j < strokePointsCount) {
                    var yBytes = getSubArray(data, currentIndex, currentIndex + 2);
                    if (yBytes[0] == -1) {
                        for (var k = 0; k < yBytes[1] - 1; k++) {
                            yArray.push(yArray[yArray.length - 1]);
                        }
                        j += k;
                    } else {
                        yArray.push(bytes2LongLong(yBytes));
                        j++;
                    }
                    currentIndex = currentIndex + 2;
                }

                j = 0;
                while (j < strokePointsCount) {
                    var tBytes = getSubArray(data, currentIndex, currentIndex + 2);
                    if (tBytes[0] == -1) {
                        for (var k = 0; k < tBytes[1] - 1; k++) {
                            tArray.push(tArray[tArray.length - 1]);
                        }
                        j += k;
                    } else {
                        tArray.push(bytes2LongLong(tBytes));
                        j++;
                    }
                    currentIndex = currentIndex + 2;
                }

                j = 0;
                for (; j < strokePointsCount; j++) {
                    points.push({
                        "x": xArray[j],
                        "y": yArray[j],
                        "t": tArray[j]
                    })
                }
                allPoints.push({
                    "points": points
                })

            }
            return JSON.stringify(allPoints);
	}

	this.getDistinguishReturn =function (handData) {
    	var base64 = new Base64();

    	var base64Data = encodeURIComponent(base64.encode(stringToByte(this.paint(handData))));
        var URL = "http://192.168.93.108:8080/PaperlessWriting/PaperlessServlet?functionType=verifyTrail";
		var xmlHttp = new XMLHttpRequest();
		xmlHttp.onreadystatechange = function () {
			if (xmlHttp.readyState === XMLHttpRequest.DONE&&xmlHttp.status === 200 ){

				var result = xmlHttp.responseText;
				var xmlDoc = loadXML(result);
				var resultNode = xmlDoc.getElementsByTagName("Result");
				var code = resultNode[0].getElementsByTagName("Code")[0].firstChild.nodeValue;
				var message = resultNode[0].getElementsByTagName("Message")[0].firstChild.nodeValue;


                if (code==="600200"){
					//alert("可能的字符："+message);
					return message;
				}else {
                	//alert("errorCode:"+code+" errorMessage:"+message);
                    return "errorCode:"+code+" errorMessage:"+message;
				}
			}
        }
        xmlHttp.open("POST",URL,true);
        xmlHttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");

        xmlHttp.send("handMessage="+base64Data);

    }

    function loadXML(xmlMessage) {
        var xmlDoc=null;
        //判断浏览器的类型
        //支持IE浏览器
        if(!window.DOMParser && window.ActiveXObject){   //window.DOMParser 判断是否是非ie浏览器
            var xmlDomVersions = ['MSXML.2.DOMDocument.6.0','MSXML.2.DOMDocument.3.0','Microsoft.XMLDOM'];
            for(var i=0;i<xmlDomVersions.length;i++){
                try{
                    xmlDoc = new ActiveXObject(xmlDomVersions[i]);
                    xmlDoc.async = false;
                    xmlDoc.loadXML(xmlMessage); //loadXML方法载入xml字符串
                    break;
                }catch(e){

                }
            }
        }
        //支持Mozilla浏览器
        else if(window.DOMParser && document.implementation && document.implementation.createDocument){
            try{
				/* DOMParser 对象解析 XML 文本并返回一个 XML Document 对象。
				 * 要使用 DOMParser，使用不带参数的构造函数来实例化它，然后调用其 parseFromString() 方法
				 * parseFromString(text, contentType) 参数text:要解析的 XML 标记 参数contentType文本的内容类型
				 * 可能是 "text/xml" 、"application/xml" 或 "application/xhtml+xml" 中的一个。注意，不支持 "text/html"。
				 */
                domParser = new DOMParser();
                xmlDoc = domParser.parseFromString(xmlMessage, 'text/xml');
            }catch(e){

            }
        }
        else{
            return null;
        }

        return xmlDoc;
    }


}

